<?php 
require_once(LIB_PATH.DS.'events-company-amsterdam-database.php');

class Category
{
	protected static $table_name="gallery_categories";
	protected static $db_fields = array('id', 'name', 'naam', 'position');
	public $id;
	public $name;
	public $naam;
	public $position;
	
	public $orig_position;
	
	// Common Database Methods	
	public static function find_all()
	{
		return self::find_by_sql("SELECT * FROM ".self::$table_name." ORDER BY position ASC");
	}
	
	public static function find_by_id($id=0)
	{
		global $database;
		$result_array = self::find_by_sql("SELECT * FROM ".self::$table_name." WHERE id={$id} LIMIT 1");
		return !empty($result_array) ? array_shift($result_array) : false;
	}
	
	public static function find_first()
	{
		global $database;
		$result_array = self::find_by_sql("SELECT * FROM ".self::$table_name." WHERE position=1 LIMIT 1");
		return !empty($result_array) ? array_shift($result_array) : false;
	}
	
	public static function find_by_sql($sql="")
	{
		global $database;
		$result_set = $database->query($sql);
		$object_array = array();
		while ($row = $database->fetch_array($result_set))
		{
			$object_array[] = self::instantiate($row);
		}
		return $object_array;
	
	}
	
	private static function instantiate($record)
	{
		// could check that $record exists and is an array
		$object = new self;
		
		// More dynamic, short-form approach:
		foreach($record as $attribute=>$value)
		{
			if($object->has_attribute($attribute))
			{
				$object->$attribute = $value;
			}
		}
		return $object;		
	}
	
	private function has_attribute($attribute)
	{
		//get_object_vars returns an associative array with all attributes 
		//(incl. private ones!!) as the keys and their current values as the value
		$object_vars = $this->attributes();
		// We don't care about the value, we just want to know if the key exists
		// Will return true or false
		return array_key_exists($attribute, $object_vars);
	}

	protected function attributes()
	{  	// return an array of attribute keys and their values
		$attributes = array();
		foreach(self::$db_fields as $field)
		{
			if(property_exists($this, $field))
			{
				$attributes[$field] = $this->$field; // use dynamic $field
			}
		}
		return $attributes; // for more complex tables can use SQL show_fields_from (list) can be parsed then build array dynamically
	}
	
	protected function sanitized_attributes()
	{
		global $database;
		$clean_attributes = array();
		foreach($this->attributes() as $key => $value)
		{
			$clean_attributes[$key] = $database->escape_value($value);
		}
		return $clean_attributes;
	}
	
	public function save()
	{
		if (isset($this->id))
		{
			$this->update();
		} else
		{
			$this->create();
		}
	}
	
	public function create()
	{
		global $database;
		
		$query = "UPDATE ".self::$table_name." SET position=position+1 WHERE position>={$this->position}";
		$database->query($query);
		
		$attributes = $this->sanitized_attributes();
		
		$sql  = "INSERT INTO ".self::$table_name." (";
		$sql .= join(", ", array_keys($attributes));
		$sql .= ") VALUES ('";
		$sql .= join("', '", array_values($attributes));
		$sql .= "')";
		
		if($database->query($sql))
		{
			$this->id = $database->insert_id();
			return true;
		} else
		{
			return false;
		}
	}
	
	public function update()
	{
		global $database;

		if ($this->orig_position<$this->position)
		{
			$this->fill_gaps();
		} elseif ($this->orig_position>$this->position)
		{
			$this->check_double();
		}

		$attributes = $this->sanitized_attributes();
		$attribute_pairs = array();
		foreach ($attributes as $key => $value)
		{
			$attribute_pairs[] = "{$key}='{$value}'";
		}
		
		$sql  = "UPDATE ".self::$table_name." SET ";
		$sql .= join(", ", $attribute_pairs);
		$sql .= " WHERE id=".$database->escape_value($this->id); //dont have to escape id, but no harm
		$database->query($sql);
		return ($database->affected_rows() >0) ? true: false;
	}
	
	public function check_double()
	{				
		global $database;

		$query = "SELECT * FROM ".self::$table_name;
		$result = $database->query($query);
		while ($row = mysql_fetch_array($result))
		{
			if ($row['position'] < $this->orig_position && $row['position'] >= $this->position)
			{
				$update = "UPDATE ".self::$table_name." SET position=position+1 WHERE id={$row['id']}";
				$database->query($update);
			}
		}
		return true;
	}
	
	public function fill_gaps()
	{				
		global $database;

		$query  = "SELECT * FROM ".self::$table_name;
		$result	= $database->query($query);
		while ($row = mysql_fetch_array($result))
		{
			if ($row['position'] > $this->orig_position && $row['position'] <= $this->position)
			{
				$shift = "UPDATE ".self::$table_name." SET position=position-1 WHERE id='{$row['id']}'";
				$database->query($shift);
			}
		}
			
			return true;
	}
	
	public function delete()
	{
		global $database;
		
		$query = "UPDATE ".self::$table_name." SET position=position-1 WHERE position>{$this->position}";
		$database->query($query);
		
		$sql  = "DELETE FROM ".self::$table_name;
		$sql .= " WHERE id=".$database->escape_value($this->id);
		$sql .= " LIMIT 1";
		$database->query($sql);
		return ($database->affected_rows() == 1) ? true : false;
	}
}

?>