<?php 
class Session
{
	private $logged_in = false;
	private $admin_user = false;
	public $user_id;
	public $user_level; // 1 = admin
	public $message;
	
	function __construct()
	{
		session_start();
		$this->check_message();
		$this->check_login();
		if($this->logged_in)
		{
			//actions to take right away if user is logged in
			$this->check_user_level();
		} else
		{
			//actions to take right away if user is not logged in
		}
	}
	
	public function is_logged_in()
	{
		return $this->logged_in;
	}
	
	public function is_admin_user()
	{
		return $this->admin_user;
	}
	
	public function login($user)
	{
		// database should find user based on username/password
		if($user)
		{
			$this->user_id = $_SESSION['user_id'] = $user->id;
			$this->logged_in = true;
			$this->set_user_level($user);
		}
	}
	
	public function set_user_level($user)
	{
		$this->user_level = $_SESSION['user_level'] = $user->level;
		if($user->level == 1)
		{
			$this->admin_user = true;
		}
	}
	
	public function logout()
	{
		unset($_SESSION['user_id']);
		unset($_SESSION['user_level']);
		unset($this->user_id);
		unset($this->user_level);
		$this->logged_in = false;
		$this->admin_user = false;
	}
	
	public function message($msg="")
	{
		if(!empty($msg))
		{	// Then this is "set message"
			// make sure you understand why $this->message=$msg wouldn't work
			$_SESSION['message'] = $msg;
		} else
		{	// then this is "get message"
			return $this->message;
		}
	}
	
	private function check_login()
	{
		if(isset($_SESSION['user_id']))
		{
			$this->user_id = $_SESSION['user_id'];
			$this->logged_in = true;
		} else
		{
			unset($this->user_id);
			$this->logged_in = false;
		}
	}
	
	private function check_user_level()
	{
		if(isset($_SESSION['user_level']))
		{
			$this->user_level = $_SESSION['user_level'];
			if($this->user_level == 1)
			{
				$this->admin_user = true;
			}
		} else
		{
			unset($this->user_level);
			$this->admin_user = false;
		}
	}
	
	private function check_message()
	{	// Is there a message stored in the session?
		if(isset($_SESSION['message']))
		{	// Add it as an attribute and erase the stored version
			$this->message = $_SESSION['message'];
			unset($_SESSION['message']);
		} else
		{
			$this->message = "";
		}
	}
}

$session = new Session();
$message = $session->message();

?>