<?php 
require_once(LIB_PATH.DS.'cape-guided-tours-database.php');

class Bank
{
	protected static $table_name="banks";
	protected static $db_fields = array('id', 'name');
	public $id;
	public $name;
	
//	 Common Database Methods	
	public static function find_all()
	{
		return self::find_by_sql("SELECT * FROM ".self::$table_name." ORDER BY name ASC");
	}
	
	public static function find_by_id($id=0)
	{
		global $database;
		$result_array = self::find_by_sql("SELECT * FROM ".self::$table_name." WHERE id={$id} LIMIT 1");
		return !empty($result_array) ? array_shift($result_array) : false;
	}
	
	public static function find_by_name($name="")
	{
		global $database;
		$result_array = self::find_by_sql("SELECT * FROM ".self::$table_name." WHERE name='{$name}' LIMIT 1");
		return !empty($result_array) ? array_shift($result_array) : false;
	}
	
	public static function find_by_sql($sql="")
	{
		global $database;
		$result_set = $database->query($sql);
		$object_array = array();
		while ($row = $database->fetch_array($result_set))
		{
			$object_array[] = self::instantiate($row);
		}
		return $object_array;
	
	}
	
	private static function instantiate($record)
	{
		// could check that $record exists and is an array
		$object = new self;
		
		foreach($record as $attribute=>$value)
		{
			if($object->has_attribute($attribute))
			{
				$object->$attribute = $value;
			}
		}
		return $object;		
	}
	
	private function has_attribute($attribute)
	{
		$object_vars = $this->attributes();
		return array_key_exists($attribute, $object_vars);
	}

	protected function attributes()
	{  	// return an array of attribute keys and their values
		$attributes = array();
		foreach(self::$db_fields as $field)
		{
			if(property_exists($this, $field))
			{
				$attributes[$field] = $this->$field; // use dynamic $field
			}
		}
		return $attributes;
	}
	
	protected function sanitized_attributes()
	{
		global $database;
		$clean_attributes = array();
		foreach($this->attributes() as $key => $value)
		{
			$clean_attributes[$key] = $database->escape_value($value);
		}
		return $clean_attributes;
	}
	
	public function create()
	{
		global $database;
		$attributes = $this->sanitized_attributes();
		
		$sql  = "INSERT INTO ".self::$table_name." (";
		$sql .= join(", ", array_keys($attributes));
		$sql .= ") VALUES ('";
		$sql .= join("', '", array_values($attributes));
		$sql .= "')";
		
		if($database->query($sql))
		{
			$this->id = $database->insert_id();
			return true;
		} else
		{
			return false;
		}
	}
	
	public function update()
	{
		global $database;
		$attributes = $this->sanitized_attributes();
		$attribute_pairs = array();
		foreach ($attributes as $key => $value)
		{
			$attribute_pairs[] = "{$key}='{$value}'";
		}
		
		$sql  = "UPDATE ".self::$table_name." SET ";
		$sql .= join(", ", $attribute_pairs);
		$sql .= " WHERE id=".$database->escape_value($this->id); //dont have to escape id, but no harm
		$database->query($sql);
		return($database->affected_rows() == 1) ? true : false;
	}
	
	public function delete()
	{
		global $database;
		$sql  = "DELETE FROM ".self::$table_name;
		$sql .= " WHERE id=".$database->escape_value($this->id);
		$sql .= " LIMIT 1";
		$database->query($sql);
		return ($database->affected_rows() == 1) ? true : false;
	}
}

class BankBranch
{
	protected static $table_name="bank_branches";
	protected static $db_fields = array('id', 'name', 'code', 'banks_id');
	public $id;
	public $name;
	public $code;
	public $banks_id;
	
//	 Common Database Methods	
	public static function find_all()
	{
		return self::find_by_sql("SELECT * FROM ".self::$table_name." ORDER BY name ASC");
	}
	
	public static function find_by_id($id=0)
	{
		global $database;
		$result_array = self::find_by_sql("SELECT * FROM ".self::$table_name." WHERE id={$id} LIMIT 1");
		return !empty($result_array) ? array_shift($result_array) : false;
	}
	
	public static function find_by_name($name="", $bank="")
	{
		global $database;
		$result_array = self::find_by_sql("SELECT * FROM ".self::$table_name." WHERE name='{$name}' AND banks_id={$bank} LIMIT 1");
		return !empty($result_array) ? array_shift($result_array) : false;
	}
	
	public static function find_by_bank($bank=0)
	{
		global $database;
		return self::find_by_sql("SELECT * FROM ".self::$table_name." WHERE banks_id={$bank} LIMIT 1");
	}
	
	public static function find_by_sql($sql="")
	{
		global $database;
		$result_set = $database->query($sql);
		$object_array = array();
		while ($row = $database->fetch_array($result_set))
		{
			$object_array[] = self::instantiate($row);
		}
		return $object_array;
	
	}
	
	private static function instantiate($record)
	{
		// could check that $record exists and is an array
		$object = new self;
		
		foreach($record as $attribute=>$value)
		{
			if($object->has_attribute($attribute))
			{
				$object->$attribute = $value;
			}
		}
		return $object;		
	}
	
	private function has_attribute($attribute)
	{
		$object_vars = $this->attributes();
		return array_key_exists($attribute, $object_vars);
	}

	protected function attributes()
	{  	// return an array of attribute keys and their values
		$attributes = array();
		foreach(self::$db_fields as $field)
		{
			if(property_exists($this, $field))
			{
				$attributes[$field] = $this->$field; // use dynamic $field
			}
		}
		return $attributes;
	}
	
	protected function sanitized_attributes()
	{
		global $database;
		$clean_attributes = array();
		foreach($this->attributes() as $key => $value)
		{
			$clean_attributes[$key] = $database->escape_value($value);
		}
		return $clean_attributes;
	}
	
	public function create()
	{
		global $database;
		$attributes = $this->sanitized_attributes();
		
		$sql  = "INSERT INTO ".self::$table_name." (";
		$sql .= join(", ", array_keys($attributes));
		$sql .= ") VALUES ('";
		$sql .= join("', '", array_values($attributes));
		$sql .= "')";
		
		if($database->query($sql))
		{
			$this->id = $database->insert_id();
			return true;
		} else
		{
			return false;
		}
	}
	
	public function update()
	{
		global $database;
		$attributes = $this->sanitized_attributes();
		$attribute_pairs = array();
		foreach ($attributes as $key => $value)
		{
			$attribute_pairs[] = "{$key}='{$value}'";
		}
		
		$sql  = "UPDATE ".self::$table_name." SET ";
		$sql .= join(", ", $attribute_pairs);
		$sql .= " WHERE id=".$database->escape_value($this->id); //dont have to escape id, but no harm
		$database->query($sql);
		return($database->affected_rows() == 1) ? true : false;
	}
	
	public function delete()
	{
		global $database;
		$sql  = "DELETE FROM ".self::$table_name;
		$sql .= " WHERE id=".$database->escape_value($this->id);
		$sql .= " LIMIT 1";
		$database->query($sql);
		return ($database->affected_rows() == 1) ? true : false;
	}
}

class AccountType
{
	protected static $table_name="account_types";
	protected static $db_fields = array('id', 'account_type');
	public $id;
	public $account_type;
	
//	 Common Database Methods	
	public static function find_all()
	{
		return self::find_by_sql("SELECT * FROM ".self::$table_name." ORDER BY account_type ASC");
	}
	
	public static function find_by_id($id=0)
	{
		global $database;
		$result_array = self::find_by_sql("SELECT * FROM ".self::$table_name." WHERE id={$id} LIMIT 1");
		return !empty($result_array) ? array_shift($result_array) : false;
	}
	
	public static function find_by_sql($sql="")
	{
		global $database;
		$result_set = $database->query($sql);
		$object_array = array();
		while ($row = $database->fetch_array($result_set))
		{
			$object_array[] = self::instantiate($row);
		}
		return $object_array;
	
	}
	
	private static function instantiate($record)
	{
		// could check that $record exists and is an array
		$object = new self;
		
		foreach($record as $attribute=>$value)
		{
			if($object->has_attribute($attribute))
			{
				$object->$attribute = $value;
			}
		}
		return $object;		
	}
	
	private function has_attribute($attribute)
	{
		$object_vars = $this->attributes();
		return array_key_exists($attribute, $object_vars);
	}

	protected function attributes()
	{  	// return an array of attribute keys and their values
		$attributes = array();
		foreach(self::$db_fields as $field)
		{
			if(property_exists($this, $field))
			{
				$attributes[$field] = $this->$field; // use dynamic $field
			}
		}
		return $attributes;
	}
	
	protected function sanitized_attributes()
	{
		global $database;
		$clean_attributes = array();
		foreach($this->attributes() as $key => $value)
		{
			$clean_attributes[$key] = $database->escape_value($value);
		}
		return $clean_attributes;
	}
	
	public function create()
	{
		global $database;
		$attributes = $this->sanitized_attributes();
		
		$sql  = "INSERT INTO ".self::$table_name." (";
		$sql .= join(", ", array_keys($attributes));
		$sql .= ") VALUES ('";
		$sql .= join("', '", array_values($attributes));
		$sql .= "')";
		
		if($database->query($sql))
		{
			$this->id = $database->insert_id();
			return true;
		} else
		{
			return false;
		}
	}
	
	public function update()
	{
		global $database;
		$attributes = $this->sanitized_attributes();
		$attribute_pairs = array();
		foreach ($attributes as $key => $value)
		{
			$attribute_pairs[] = "{$key}='{$value}'";
		}
		
		$sql  = "UPDATE ".self::$table_name." SET ";
		$sql .= join(", ", $attribute_pairs);
		$sql .= " WHERE id=".$database->escape_value($this->id); //dont have to escape id, but no harm
		$database->query($sql);
		return($database->affected_rows() == 1) ? true : false;
	}
	
	public function delete()
	{
		global $database;
		$sql  = "DELETE FROM ".self::$table_name;
		$sql .= " WHERE id=".$database->escape_value($this->id);
		$sql .= " LIMIT 1";
		$database->query($sql);
		return ($database->affected_rows() == 1) ? true : false;
	}
}

?>