<?php 
require_once(LIB_PATH.DS.'cape-guided-tours-database.php');

class Page
{
	protected static $table_name="pages";
	protected static $db_fields = array('id', 'editable', 'name', 'pagename', 'content');
	public $id;
	public $editable;
	public $name;
	public $pagename;
	public $content;
	
//	 Common Database Methods	
	public static function find_all()
	{
		return self::find_by_sql("SELECT * FROM ".self::$table_name." ORDER BY name ASC");
	}
	
	public static function find_by_id($id=0)
	{
		global $database;
		$result_array = self::find_by_sql("SELECT * FROM ".self::$table_name." WHERE id={$id} LIMIT 1");
		return !empty($result_array) ? array_shift($result_array) : false;
	}
	
	public static function find_all_editable()
	{
		global $database;
		return self::find_by_sql("SELECT * FROM ".self::$table_name." WHERE editable = 1 ORDER BY name ASC");
	}
	
	public static function find_by_pagename($pagename = "")
	{
		//global $database;
		//return self::find_by_sql("SELECT * FROM ".self::$table_name." WHERE pagename = '{$pagename}' ORDER BY name ASC");
		
		global $database;
		$result_array = self::find_by_sql("SELECT * FROM ".self::$table_name." WHERE pagename='{$pagename}' LIMIT 1");
		return !empty($result_array) ? array_shift($result_array) : false;
	}
	
	public static function search($name="")
	{
		global $database;
		return self::find_by_sql("SELECT * FROM ".self::$table_name." WHERE name LIKE %'{$name}'%");
	}
	
	public static function find_by_sql($sql="")
	{
		global $database;
		$result_set = $database->query($sql);
		$object_array = array();
		while ($row = $database->fetch_array($result_set))
		{
			$object_array[] = self::instantiate($row);
		}
		return $object_array;
	
	}
	
	private static function instantiate($record)
	{
		// could check that $record exists and is an array
		$object = new self;
		
		foreach($record as $attribute=>$value)
		{
			if($object->has_attribute($attribute))
			{
				$object->$attribute = $value;
			}
		}
		return $object;		
	}
	
	private function has_attribute($attribute)
	{
		$object_vars = $this->attributes();
		return array_key_exists($attribute, $object_vars);
	}

	protected function attributes()
	{  	// return an array of attribute keys and their values
		$attributes = array();
		foreach(self::$db_fields as $field)
		{
			if(property_exists($this, $field))
			{
				$attributes[$field] = $this->$field; // use dynamic $field
			}
		}
		return $attributes;
	}
	
	protected function sanitized_attributes()
	{
		global $database;
		$clean_attributes = array();
		foreach($this->attributes() as $key => $value)
		{
			$clean_attributes[$key] = $database->escape_value($value);
		}
		return $clean_attributes;
	}
	
	public function create()
	{
		global $database;
		$this->add_rewrite_rule();
		$attributes = $this->sanitized_attributes();
		
		$sql  = "INSERT INTO ".self::$table_name." (";
		$sql .= join(", ", array_keys($attributes));
		$sql .= ") VALUES ('";
		$sql .= join("', '", array_values($attributes));
		$sql .= "')";
		
		if($database->query($sql))
		{
			$this->id = $database->insert_id();
			$this->add_rewrite_rule();
			return true;
		} else
		{
			return false;
		}
	}
	
	public function update()
	{
		global $database;
		$attributes = $this->sanitized_attributes();
		$attribute_pairs = array();
		foreach ($attributes as $key => $value)
		{
			$attribute_pairs[] = "{$key}='{$value}'";
		}
		
		$sql  = "UPDATE ".self::$table_name." SET ";
		$sql .= join(", ", $attribute_pairs);
		$sql .= " WHERE id=".$database->escape_value($this->id); //dont have to escape id, but no harm
		$database->query($sql);
		return($database->affected_rows() == 1) ? true : false;
	}
	
	public function delete()
	{
		global $database;
		
		$this->remove_rewrite_rule();
		
		$sql  = "DELETE FROM ".self::$table_name;
		$sql .= " WHERE id=".$database->escape_value($this->id);
		$sql .= " LIMIT 1";
		$database->query($sql);
		return ($database->affected_rows() == 1) ? true : false;
	}
	
	protected function add_rewrite_rule()
	{
		global $database;
		$file = SITE_ROOT.DS.'.htaccess';
		
		$content="";
	 	$text = "";
		
	  	$text .= "RewriteRule ^";
	  	$text .= str_replace(".html","\.html",$this->pagename);
	  	$text .= "$ tour-cape-town.php?id=";
	  	$text .= $this->id;
	  	$text .= "\n";
	  	
	  	$string = file($file);
	  	$count = count($string);
		for($i=0;$i<$count-1;$i++)
		{
			$content .= $string[$i];
		}
		$content .= $text;
		$content .= 'RewriteRule ^(.*)\.html(.*)$ $1\.php$2';
	
		if ($handle = fopen($file,'w+'))
		{	
			fwrite($handle, $content);
			fclose($handle);
			return true;
		} else 
		{ 
			global $session;
			$session->message = "Could not open .htaccess file for writing.";
			return false;
		}
 	}
 	
	protected function remove_rewrite_rule()
	{
		$file = SITE_ROOT.DS.'.htaccess';
		$old_rule = "RewriteRule ^";
	  	$old_rule .= str_replace(".html","\.html",$this->pagename);
	  	$old_rule .= "$ tour-cape-town.php?id=";
	 	$old_rule .= $this->id;
		
		$string = file($file);
		$content ="";
		for($i = 0; $i < count($string); $i++)
		{
			if(strstr($string[$i],$old_rule))
			{
				$content .= "";
			}
			else
			{
				$content .= $string[$i];
			}
		}
	 	
		if ($handle = fopen($file,'w'))
		{
			if (fwrite($handle,$content))
			{
				fclose($handle);
				return true;
			}
		} else 
		{
			global $session;
			$session->message ="Error opening file.";
		}
 	}

}

?>