<?php 

require_once(LIB_PATH.DS.'website-cape-town-database.php');

class Photograph
{
	protected static $table_name="images";
	protected static $db_fields = array('id', 'page_id', 'filename', 'size', 'position');
	public $id;
	public $page_id;
	public $filename;
	public $size;
	public $position;	
	
	private $temp_path;
	protected $upload_dir="web-design-cape-town-images";
	public $errors=array();
	
	protected $upload_errors = array
	( 		// http://www.php.net/manual/en/features.file-upload.errors.php
		UPLOAD_ERR_OK			=> "No errors.",
		UPLOAD_ERR_INI_SIZE		=> "Larger than upload_max_filesize.",
		UPLOAD_ERR_FORM_SIZE	=> "Larger than form max_file_size.",
		UPLOAD_ERR_PARTIAL		=> "Partial upload.",
		UPLOAD_ERR_NO_FILE		=> "No file.",
		UPLOAD_ERR_NO_TMP_DIR	=> "No temporary directory.",
		UPLOAD_ERR_CANT_WRITE	=> "Can't write to disk.",
		UPLOAD_ERR_EXTENSION 	=> "File upload stopped by extension.",
	);
	
	// Pass in $_FILE(['uploaded_file']) as an argument
	public function attach_file($file)
	{
		// Perform error checking on the form parameters
		if(!$file || empty($file) || !is_array($file))
		{	// error: nothing uploaded or wrong argument usage
			$this->errors[] = "No file was uploaded.";
			return false;
		} elseif($file['error'] !=0)
		{	// error: report what PHP says went wrong
			$this->errors[] = $this->upload_errors[$file['error']];
			return false;
		} else
		{
			// Set object attributes to the form parameters.
			$this->temp_path = $file['tmp_name'];
			
			if ($this->page_id==1)
				{ $this->filename="website-design-cape-town-";}
			elseif ($this->page_id==2)
				{ $this->filename="web-design-cape-town-";}
			elseif ($this->page_id==3)
				{ $this->filename="online-marketing-cape-town-";}
			elseif ($this->page_id==4)
				{ $this->filename="graphic-design-cape-town-";}
			elseif ($this->page_id==5)
				{ $this->filename="corporate-id-cape-town-";}
			elseif ($this->page_id==6)
				{ $this->filename="web-site-design-cape-town-";}
			
			$this->filename		.= basename($file['name']);
			$this->size			= $file['size'];
			// Don't worry about saving anything to database yet			
			return true;
		}
	}
	
	public function save()
	{	
		if (isset($this->id))
		{
			$this->update();
		} else
		{		
			// Make sure there are no errors. 
			// Can't save if there are pre-existing errors
			if(!empty($this->errors)) {return false;}
			
			// Can't save without filename and temp location
			if(empty($this->filename) || empty($this->temp_path))
			{
				$this->errors[] = "The file location was not available.";
				return false;
			}
			
			
			// Determine the target_path
			$target_path = SITE_ROOT.DS.$this->upload_dir.DS.$this->filename;
			
			// Make sure file doesn't already exist in the target location
			if(file_exists($target_path))
			{
				$this->errors[] = "The file {$this->filename} already exists.";
				return false;
			}
			
			// Attempt to move the file
			if(move_uploaded_file($this->temp_path, $target_path))
			{	// Success
				// Save a corresponding entry to database
				if($this->create())
				{
					unset($this->temp_path);
					return true;
				}
			
			} else
			{	// File was not moved
				$this->errors[] = "The file upload failed, possibly due to incorrect permissions on the upload folder.";
				return false;
			}
			
			// Save a corresponding entry to the database
			$this->create();
		}
	
	}
	
	public function destroy()
	{	// First remove db entry
		if($this->delete())
		{
			// then remove the file
			// Note that even though the database entry is gone, 
			// this object is still around (which lets us use $this->image_path()).
			$target_path = SITE_ROOT.DS.$this->image_path();
			return unlink($target_path) ? true : false;
		} else
		{
			// database delete failed
			return false;
		}
	}
	
	public function image_path()
	{
		return $this->upload_dir.'/'.$this->filename;
	}
	
	public function size_as_text()
	{
		if($this->size < 1024)
		{
			return "{$this->size} bytes";
		} elseif($this->size < 1048576)
		{
			$size_kb = round($this->size/1024);
			return "{$size_kb} KB";
		} else
		{
			$size_mb = round($this->size/1048576, 1);
			return "{$size_mb} MB";
		}
	}
	
	// Common Database Methods	
	public static function find_all()
	{
		return self::find_by_sql("SELECT * FROM ".self::$table_name);
	}
	
	public static function find_by_id($id="")
	{
		global $database;
		$result_array = self::find_by_sql("SELECT * FROM ".self::$table_name." WHERE id=".$database->escape_value($id)." LIMIT 1");
		return !empty($result_array) ? array_shift($result_array) : false;
	}
	
	public static function get_photos_for_page($page_id)
	{
			global $database;	
			$query  = "SELECT * FROM ".self::$table_name." WHERE page_id=".$page_id." ORDER BY position ASC";
			$photo_set = self::find_by_sql($query);
			return $photo_set;
	}
	
	public static function find_by_sql($sql="")
	{
		global $database;
		$result_set = $database->query($sql);
		$object_array = array();
		while ($row = $database->fetch_array($result_set))
		{
			$object_array[] = self::instantiate($row);
		}
		return $object_array;
	}
	
	private static function instantiate($record)
	{
		// could check that $record exists and is an array
		$object = new self;

		foreach($record as $attribute=>$value)
		{
			if($object->has_attribute($attribute))
			{
				$object->$attribute = $value;
			}
		}
		return $object;		
	}
	
	private function has_attribute($attribute)
	{
		//get_object_vars returns an associative array with all attributes 
		//(incl. private ones!!) as the keys and their current values as the value
		$object_vars = $this->attributes();
		// We don't care about the value, we just want to know if the key exists
		// Will return true or false
		return array_key_exists($attribute, $object_vars);
	}

	protected function attributes()
	{  	// return an array of attribute keys and their values
		$attributes = array();
		foreach(self::$db_fields as $field)
		{
			if(property_exists($this, $field))
			{
				$attributes[$field] = $this->$field; // use dynamic $field
			}
		}
		return $attributes; // for more complex tables can use SQL show_fields_from (list) can be parsed then build array dynamically
	}
	
	protected function sanitized_attributes()
	{
		global $database;
		$clean_attributes = array();
		foreach($this->attributes() as $key => $value)
		{
			$clean_attributes[$key] = $database->escape_value($value);
		}
		return $clean_attributes;
	}

	public function create()
	{
		global $database;
		
		$query = "UPDATE ".self::$table_name." SET position=position+1 WHERE page_id={$this->page_id} AND position>={$this->position}";
		$database->query($query);
		
		$attributes = $this->sanitized_attributes();
		
		$sql  = "INSERT INTO ".self::$table_name." (";
		$sql .= join(", ", array_keys($attributes));
		$sql .= ") VALUES ('";
		$sql .= join("', '", array_values($attributes));
		$sql .= "')";
		
		if($database->query($sql))
		{
			$this->id = $database->insert_id();
			return true;
		} else
		{
			return false;
		}
	}
	
	public function update()
	{
		global $database;
		global $orig_position;

		if ($orig_position<$this->position)
		{
			$this->fill_gaps();
		} elseif ($orig_position>$this->position)
		{
			$this->check_double();
		}

		$attributes = $this->sanitized_attributes();
		$attribute_pairs = array();
		foreach ($attributes as $key => $value)
		{
			$attribute_pairs[] = "{$key}='{$value}'";
		}
		
		$sql  = "UPDATE ".self::$table_name." SET ";
		$sql .= join(", ", $attribute_pairs);
		$sql .= " WHERE id=".$database->escape_value($this->id); //dont have to escape id, but no harm
		$database->query($sql);
		return ($database->affected_rows() >0) ? true: false;
	}
	
	public function check_double()
	{				
		global $database;
		global $orig_position;

		$query = "SELECT * FROM ".self::$table_name." WHERE page_id=".$this->page_id;
		$result = $database->query($query);
		while ($row = mysql_fetch_array($result))
		{
			if ($row['position'] < $orig_position && $row['position'] >= $this->position)
			{
				$update = "UPDATE ".self::$table_name." SET position=position+1 WHERE id={$row['id']}";
				$database->query($update);
			}
		}
		return true;
	}
	
	public function fill_gaps()
	{				
		global $database;
		global $orig_position;

		$query  = "SELECT * FROM ".self::$table_name." WHERE page_id=".$this->page_id;
		$result	= $database->query($query);
		while ($row = mysql_fetch_array($result))
		{
			if ($row['position'] > $orig_position && $row['position'] <= $this->position)
			{
				$shift = "UPDATE ".self::$table_name." SET position=position-1 WHERE id='{$row['id']}'";
				$database->query($shift);
			}
		}
			
			return true;
	}
	
	public function delete()
	{
		global $database;
		
		$query = "UPDATE ".self::$table_name." SET position=position-1 WHERE page_id={$this->page_id} AND position>{$this->position}";
		$database->query($query);
		
		$sql  = "DELETE FROM ".self::$table_name;
		$sql .= " WHERE id=".$database->escape_value($this->id);
		$sql .= " LIMIT 1";
		$database->query($sql);
		return ($database->affected_rows() == 1) ? true : false;
	}
}

?>
